@@ -11,6 +11,9 @@ void dust();
 
 void dust_rect() {}
 
+void set_dust_level(int value);
+void getfafun(char *name, void (*fun)());
+
 /*void main(argc, argv)
 int argc;
 char *argv[];
