@@ -120,7 +120,7 @@ int get_trace_item
  * matrix is allocated to accommodate the longest sequence in the set.
  **************************************************************************/
 static PROB_T compute_forward_score
-  (BOOLEAN_T local,      // Was local scoring performed?
+  (bool local,      // Was local scoring performed?
    int       seq_length, // Number of columns in the matrix.
    MATRIX_T* forward_matrix)
 {
@@ -203,7 +203,7 @@ static void set_seq_length
    MATCH_T*  this_match)    // [In/Out] Sequence-to-model match.
 {
   // Make sure the object exists.
-  myassert(TRUE, this_match != NULL, "Tried to access null match object.");
+  myassert(true, this_match != NULL, "Tried to access null match object.");
 
   // Make sure the match object is big enough to hold this sequence.
   if (get_array_length(this_match->trace) < seq_length+1) {
@@ -226,7 +226,7 @@ static void get_match
   (int       end_row,       // Row position of end of match.
    int       end_col,       // Column position of end of match.
    MHMM_T*   the_log_hmm,   // Needed for motif widths.
-   BOOLEAN_T motif_scoring, // Is motif-scoring in effect?
+   bool motif_scoring, // Is motif-scoring in effect?
    MATRIX_T* trace_matrix,  // Matrix of prev state indices.
    MATCH_T*  this_match)    // [In/Out] Sequence-to-model match.
 {
@@ -332,17 +332,16 @@ int find_last_unmatched_position
  *
  * Returns: Boolean indicating whether match is found.
  **************************************************************************/
-BOOLEAN_T find_next_match
-  (BOOLEAN_T is_complete,       // This match is versus a complete sequence.
+bool find_next_match
+  (bool is_complete,       // This match is versus a complete sequence.
    int       start_pos,         // Start looking for match here.
-   int       align_width,       // Width of alignment lines.
    MATRIX_T* dp_matrix,
    MHMM_T*   the_log_hmm,   	// Needed for counting motif hits.
    MATCH_T*  complete_match,    // [In] Match against entire sequence.
    MATCH_T*  partial_match)     // [Out] Local match extracted from complete.
 {
-  BOOLEAN_T in_match = FALSE;
-  BOOLEAN_T match_found = FALSE;
+  bool in_match = false;
+  bool match_found = false;
   int       start_of_match = -1;
   int       end_of_match = -1;
   int       i_row;
@@ -359,7 +358,7 @@ BOOLEAN_T find_next_match
 
     // Start of match?
     if (i_row != 0 && !in_match) {
-      in_match = TRUE;
+      in_match = true;
       start_of_match = i_col;
     }
 
@@ -367,7 +366,7 @@ BOOLEAN_T find_next_match
     else if (in_match && 
 	     (i_row == 0 || (is_complete && i_col == num_cols-1))) {
       end_of_match = i_col-1;
-      match_found = TRUE;
+      match_found = true;
       break;
     }
 
@@ -379,14 +378,14 @@ BOOLEAN_T find_next_match
 
   // If no complete match was found, quit.
   if (!match_found) {
-    if (MATCH_DEBUG && (in_match == TRUE)) {
+    if (MATCH_DEBUG && (in_match == true)) {
       fprintf(stderr, "Found partial match from %d.\n", start_of_match);
     }
-    return(FALSE);
+    return(false);
   }
-  myassert(TRUE, start_of_match != -1, "Start of match not found.");
-  myassert(TRUE, end_of_match != -1, "End of match not found.");
-  myassert(TRUE, start_of_match <= end_of_match,
+  myassert(true, start_of_match != -1, "Start of match not found.");
+  myassert(true, end_of_match != -1, "End of match not found.");
+  myassert(true, start_of_match <= end_of_match,
 	   "Start of match (%d) is after end of match (%d).",
 	   start_of_match, end_of_match);
 
@@ -418,13 +417,13 @@ BOOLEAN_T find_next_match
   }
 
   // Initialize to -1's only the part of the trace that might be printed.
-  for (i_col = MAX(0, start_of_match - align_width); 
+  for (i_col = MAX(0, start_of_match); 
        i_col < start_of_match; 
        i_col++) {
     set_array_item(i_col, -1, partial_match->trace);
   }
   for (i_col = end_of_match + 1; 
-       i_col <= MIN(end_of_match + align_width + 1, 
+       i_col <= MIN(end_of_match + 1, 
 		    complete_match->seq_length - 2); 
        i_col++) {
     set_array_item(i_col, -1, partial_match->trace);
@@ -436,7 +435,7 @@ BOOLEAN_T find_next_match
 		   partial_match->trace);
   }
   
-  return(TRUE);
+  return(true);
 }
 
 /**************************************************************************
@@ -446,15 +445,15 @@ BOOLEAN_T find_next_match
  * The sequence score and match are returned in this_match, which must
  * be pre-allocated.
  *
- * Returns TRUE iff the score is non-zero.
+ * Returns true iff the score is non-zero.
  **************************************************************************/
-BOOLEAN_T traceback_and_scoring
+bool traceback_and_scoring
   (SEARCH_T  search_type,    // What kind of search was performed?
-   BOOLEAN_T compute_trace,  // Save path?
+   bool compute_trace,  // Save path?
    int       num_states,     // Number of rows in matrix.
    int       seq_length,     // Number of columns in matrix.
    MHMM_T*   the_log_hmm,    // Needed for motif widths.
-   BOOLEAN_T motif_scoring,  // Is motif-scoring in effect?
+   bool motif_scoring,  // Is motif-scoring in effect?
    MATRIX_T* dp_matrix,      // DP matrix.
    MATRIX_T* trace_matrix,   // Corresponding traceback matrix.
    MATCH_T*  this_match)     // [Out] Trace for this match.
@@ -485,11 +484,11 @@ BOOLEAN_T traceback_and_scoring
 
   // Compute column sums for total probability scoring.
   case GLOBAL_TOTAL_SEARCH :
-    this_match->score = compute_forward_score(FALSE, seq_length, dp_matrix);
+    this_match->score = compute_forward_score(false, seq_length, dp_matrix);
     break;
 
   case LOCAL_TOTAL_SEARCH :
-    this_match->score = compute_forward_score(TRUE, seq_length, dp_matrix);
+    this_match->score = compute_forward_score(true, seq_length, dp_matrix);
     break;
 
   case INVALID_SEARCH :
@@ -499,7 +498,7 @@ BOOLEAN_T traceback_and_scoring
   // No path was found.
   if (this_match->score  <= LOG_SMALL) {
     this_match->score = LOG_ZERO;
-    return(FALSE);
+    return(false);
   }
 
   // Store the sequence length.
@@ -527,7 +526,7 @@ BOOLEAN_T traceback_and_scoring
     );
     if (first_state == last_state) {
       this_match->score = LOG_ZERO;
-      return(FALSE);
+      return(false);
     }
   }
 
@@ -537,5 +536,5 @@ BOOLEAN_T traceback_and_scoring
     this_match->end_match = seq_length - 2; // Subtract spaces on either end.
   }
 
-  return(TRUE);
+  return(true);
 }
