@@ -2,8 +2,10 @@
 #include <assert.h>
 #include <math.h>
 
+#define RCHUNK 100
+
 /* local functions */
-static BOOLEAN read_psp(
+static bool read_psp(
   FILE *data_file,              /* file containing PSP data */
   char **sample_name,           /* unique identifier of saple == positional prior */
   int *w,			/* PSP motif width */
@@ -17,12 +19,12 @@ static long read_psp_data(
 );
 static char *numberstring(
   FILE *data_file,		/* data file */
-  BOOLEAN point,		/* allow '.' at start? */
+  bool point,		/* allow '.' at start? */
   char *name,			/* name of sequence */
-  BOOLEAN skipToEOL             /* if TRUE skip rest of line */
+  bool skipToEOL             /* if true skip rest of line */
 );
-static BOOLEAN all_zeros (
-  BOOLEAN forward,
+static bool all_zeros (
+  bool forward,
   double* data_end,
   int N
 );
@@ -65,12 +67,12 @@ POSPRIORS format:
 
 */
 /**********************************************************************/
-extern void read_psp_file(
+void read_psp_file(
   char *psp_filename,		// PSP file name
   DATASET *dataset,		// the dataset
-  BOOLEAN psp_revcomp, 		// PSP file contains both strands
-  BOOLEAN meme_revcomp,		// MEME using both strands
-  MTYPE mtype			// model type
+  bool psp_revcomp, 		// PSP file contains both strands
+  bool meme_revcomp,		// MEME using both strands
+  MOTYPE mtype			// model type
 )
 {
   int i, new_w;
@@ -79,7 +81,8 @@ extern void read_psp_file(
   char *sample_name = NULL;
   double *pospriors = NULL;
   int N = 0;
-  BOOLEAN first_record = TRUE;
+  bool first_record = true;
+  double *psp_original_buf = NULL;
 
   FILE *prior_file = fopen(psp_filename,"r");
   if (!prior_file) {
@@ -100,7 +103,7 @@ extern void read_psp_file(
     // Check that width hasn't changed.
     if (first_record) {
       dataset->psp_w = new_w;
-      first_record = FALSE;
+      first_record = false;
     } else if (new_w != dataset->psp_w) {
       fprintf(stderr, "PSP width W: `%d' in PSP record with ID: `%s'.\n"
         "doesn't match initial record's W: `%d'.\n", new_w, sample_name, dataset->psp_w);
@@ -111,7 +114,6 @@ extern void read_psp_file(
     if ((sample=get_sample_by_name(sample_name)) == NULL) {
       fprintf(stderr, "Warning: PSP record ID: `%s' doesn't match any sequence's ID.\n",
         sample_name);
-      //exit(1);
       continue;
     }
 
@@ -125,23 +127,26 @@ extern void read_psp_file(
 
     // shift -psp prior right and add zero at start
     if (!psp_revcomp) {
-      double *psp = (double *) mymalloc((length+1) * (int) sizeof(double));
+      double *psp = (double *) mm_malloc((length+1) * (int) sizeof(double));
       psp[0] = 0;
       for (i=0; i<length; i++) psp[i+1] = pospriors[i];
       myfree(pospriors);
       pospriors = psp;
+      psp_original_buf = psp;
       length += 1;
     }
 
-    // Check that all entries in prior are in [0,1].
+    // Check that all entries in prior are in (0,1].
     // Check that double-stranded prior is symmetrical.
     // Check that prior sums to 0<sum<=1.
     double psp_sum = 0;
-    for (i=0; i<length; i++) {
+    double fudge = 1e-16;
+    for (i=1; i<length; i++) {
       psp_sum += pospriors[i];
-      if (pospriors[i] < 0 || pospriors[i]>1) {		// Illegal entry?
+      if (pospriors[i] == 0) pospriors[i] += fudge;	// Fudge factor
+      if (pospriors[i] <= 0 || pospriors[i]>1) {	// Illegal entry?
 	fprintf(stderr, "Illegal value (%f) in PSP record %s at position %d.\n"
-	  "All entries must be in range [0..1]\n", pospriors[i], sample_name, i);
+	  "All entries must be in range (0..1]\n", pospriors[i], sample_name, i);
         exit(1);
       }
       if (psp_revcomp && i < length/2) {		// Non-symmetrical?
@@ -169,9 +174,19 @@ extern void read_psp_file(
     //
     length = sample->length;
 
+    // Create log PSP arrays for this sample.
+    if (meme_revcomp) {
+      sample->log_psp_buf = mm_malloc(sizeof(double) * (2*length+1));
+      sample->log_psp = (sample->log_psp_buf)+length;
+    } else {
+      sample->log_psp_buf = mm_malloc(sizeof(double) * (length+1));
+      sample->log_psp = sample->log_psp_buf;
+    }
+
     // Add priors for negative strand if needed, scaled by 1/2.  
     if (meme_revcomp && !psp_revcomp) {
-      double *psp = length + (double *) mymalloc((2*length+1) * (int) sizeof(double));
+      double *psp = length + (double *) mm_malloc((2*length+1) * (int) sizeof(double));
+      psp_original_buf = psp - length;
       psp[0] = pospriors[0]; 	// middle element is now probability of no site
       for (i=1; i<=length; i++) psp[i] = psp[-i] = pospriors[i]/2;
       myfree(pospriors);	// replace pospriors with psp
@@ -187,6 +202,7 @@ extern void read_psp_file(
     }
 
     // Set 0 index of psp_original to P_{i,0}
+    sample->psp_original_buf = psp_original_buf;
     if (meme_revcomp && psp_revcomp) { 	// index from middle
       sample->psp_original = &(pospriors[length]);
     } else {	// index from left end or middle set already
@@ -235,11 +251,11 @@ extern void read_psp_file(
 
  */
 /**********************************************************************/
-extern void psp_renormalize(
+void psp_renormalize(
   DATASET *dataset,		/* the dataset */
   int new_w,			/* new motif width */
-  BOOLEAN revcomp,		/* reverse complement? */
-  MTYPE mtype			/* OOPS, ZOOPS or TCM? */
+  bool revcomp,		/* reverse complement? */
+  MOTYPE mtype			/* OOPS, ZOOPS or TCM? */
 )
 {
   // check for improper/unneccesary call
@@ -257,6 +273,8 @@ extern void psp_renormalize(
     int length = sample->length;		// sequence length
     int new_m = length-new_w+1;			// number of possible new positions
 
+    if (new_m < 1) continue;                    // skip sequence too short for motif
+
     // compute log_psp for this sequence
     if (psp_original == NULL) {			// Create a uniform prior.
       int ns = revcomp ? 2*new_m : new_m;	// possible sites on 1 or 2 strands
@@ -319,6 +337,7 @@ extern void psp_renormalize(
         psp_sum += log_psp[j];
         if (revcomp) psp_sum += log_psp[-j];
       }
+      assert(psp_sum > 0);
       // set scale factor
       double factor = (1 - psp_original[0]) / psp_sum;
       sample->max_log_psp = LOG(0);		
@@ -351,11 +370,11 @@ extern void psp_renormalize(
 	add_psp_to_log_not_o
 */
 /**********************************************************************/
-extern void add_psp_to_log_not_o(
+void add_psp_to_log_not_o(
   DATASET *dataset,		/* the dataset */
   int w,			/* motif width */
-  BOOLEAN invcomp,		/* reverse complement? */
-  MTYPE mtype 			/* model type */
+  bool invcomp,		/* reverse complement? */
+  MOTYPE mtype 			/* model type */
 )
 {
   int i,j;
@@ -368,7 +387,7 @@ extern void add_psp_to_log_not_o(
   for (i=0; i<n_samples; i++){		/* sequence */
     SAMPLE *s = samples[i];
     int lseq = s->length;
-    double *weights = s->weights;       /* prb not in a previous site */
+    WEIGHTS_T *weights = s->weights;       /* prb not in a previous site */
     double *not_o = s->not_o;		/* prb not overlapping a site */
     int *log_not_o = s->log_not_o;	/* log prb not overlapping a site */
     double max_log_psp = s->max_log_psp;	/* max pos-specific prior prob */
@@ -401,7 +420,7 @@ extern void add_psp_to_log_not_o(
  read_psp
  
  Read position-specific priors for a single sequence from the data file.
- Returns FALSE on EOF or bad sequence data.
+ Returns false on EOF or bad sequence data.
  
  If PSP format, expect length to match equivalently named sequence length.
  If PSP2 format, expect length = 2*sequence length.
@@ -419,7 +438,7 @@ extern void add_psp_to_log_not_o(
  */
 /**********************************************************************/
 
-static BOOLEAN read_psp(
+static bool read_psp(
   FILE *psp_file,		/* file containing PSP */
   char **sample_name,           /* unique ID of sample == positional prior */
   int *w,                       /* width of motifs in prior */
@@ -430,7 +449,7 @@ static BOOLEAN read_psp(
   int i, c;
   char *name = NULL;
 
-  *length = -1;			// In case of error, return TRUE with illegal length set.
+  *length = -1;			// In case of error, return true with illegal length set.
 
   /* skip anything until first positional prior name */
   c = ' '; 
@@ -440,7 +459,7 @@ static BOOLEAN read_psp(
     } 
     Skip_eol(c, psp_file);		/* go to end of line */
   }
-  if (c==EOF) return FALSE;		/* no more sequences */
+  if (c==EOF) return false;		/* no more sequences */
   
   /* get the sample name */
   /* read to first blank/tab/ or end of line/file */
@@ -463,14 +482,14 @@ static BOOLEAN read_psp(
   if (c != '\n' && c != '\r' && c != EOF) { 
     char *number;
     Skip_whi(c, psp_file);		/* skip whitespace to W */
-    number = numberstring(psp_file, FALSE, name, TRUE); // ignore anything after W
+    number = numberstring(psp_file, false, name, true); // ignore anything after W
 
     // Check that W is a number and is legal (>0)
     if (number==NULL || sscanf(number, "%d", w) != 1 || *w <= 0) {
       fprintf(stderr, "\nIllegal value for W: `%s' in PSP record with ID: `%s'.\n", 
         number, name);
       if (number != NULL) myfree(number);
-      return(TRUE);			// bad W
+      return(true);			// bad W
     }
   }
   
@@ -480,12 +499,12 @@ static BOOLEAN read_psp(
   /* sequence had bad data */
   if (*length < 0) {
     myfree(name);
-    return FALSE;			// bad prior data format
+    return false;			// bad prior data format
   }
   
   *sample_name = name;
   
-  return TRUE;
+  return true;
 } /* read_psp */
 
 /**********************************************************************/
@@ -526,7 +545,7 @@ static long read_psp_data(
       break;
     /* start of number; allow a '.' at start */		
     // want everything on the line this time
-    number = numberstring (psp_file, TRUE, name, FALSE); 
+    number = numberstring (psp_file, true, name, false); 
     if (!number) {
       fprintf(stderr, "\nUnable to find valid number in PSP sequence %ld.\n", N+1);
       fprintf(stderr, "Fix PSP file.\n");
@@ -564,9 +583,9 @@ static long read_psp_data(
 /**********************************************************************/
 static char *numberstring(
   FILE *data_file,		/* data file */
-  BOOLEAN point,		/* allow '.' at start? */
+  bool point,		/* allow '.' at start? */
   char *name,			/* name of sequence */
-  BOOLEAN skipToEOL             /* if TRUE skip rest of line */
+  bool skipToEOL             /* if true skip rest of line */
 )
 {
   char *number,
@@ -611,12 +630,12 @@ static char *numberstring(
 /*
  all_zeros
  
- TRUE if the given range of the data is all zeros, false otherwise.
+ true if the given range of the data is all zeros, false otherwise.
  
  */
 /**********************************************************************/
-static BOOLEAN all_zeros (
-  BOOLEAN forward,
+static bool all_zeros (
+  bool forward,
   double* data_end,
   int N
 )
@@ -626,13 +645,13 @@ static BOOLEAN all_zeros (
     if (data_end[(forward? j : -j)] != 0.0) {
       int k, m;
       fprintf(stderr,"all_zeros: forward = %s j = %d, N = %d, data_end[(forward? j : -j)] = %lf\n",
-	      (forward?"TRUE":"FALSE"), j, N, data_end[(forward? j : -j)]);
+	      (forward?"true":"false"), j, N, data_end[(forward? j : -j)]);
       for (k = 0, m = 0; k < N; k++, m--)
 	fprintf(stderr,"data_end[%d] = %lf ",
 		(forward? m : -m), data_end[(forward? m : -m)]);
       fprintf(stderr,"\n");
-      return FALSE;
+      return false;
     }
-  return TRUE;
+  return true;
 } /* all_zeros */
 
