@@ -18,20 +18,20 @@
  * Read a set of floats from the specified column of a tab-delimited
  * file, skipping a specified number of header lines.
  *************************************************************************/
-#define MAX_SCORES 1000000         // Maximum allowed number of input lines.
-#define MAX_LINE 1000              // Longest allowed input line.
-static ARRAY_T* read_scores_from_column
+#define MAX_SCORES 100000000         // Maximum allowed number of input lines.
+#define MAX_LINE 1000                // Longest allowed input line.
+ARRAY_T* read_scores_from_column
   (char*          score_filename,    // Input filename
    int            num_header_lines,  // Number of lines to skip at start.
-   BOOLEAN_T      print_header,      // Print the header to stdout?
    int            score_column,      // Column index (indexed from 0)
+   STRING_LIST_T* header_lines,      // Store header lines, if not NULL.
    STRING_LIST_T* input_lines        // Store file contents, if not NULL.
    )
 {
 
   // Read the p-values into a very large array.
   FILE* score_file = NULL;
-  if (open_file(score_filename, "r", TRUE, "scores", "scores",
+  if (open_file(score_filename, "r", true, "scores", "scores",
 		&score_file) == 0) {
     exit(1);
   }
@@ -47,13 +47,13 @@ static ARRAY_T* read_scores_from_column
 
     // Replace the EOL.
     int length = strlen(line);
-    if (length > 0) {
+    if (length > 0 && line[length - 1] == '\n') {
       line[length - 1] = '\0';
     }
 
     // Print the line, if requested.
-    if (print_header) {
-      printf("%s\tq-value\n", line);
+    if (header_lines != NULL) {
+      add_string(line, header_lines);
     }
 
     i_header++;
@@ -66,7 +66,7 @@ static ARRAY_T* read_scores_from_column
 
     // Replace the EOL.
     int length = strlen(line);
-    if (length > 0) {
+    if (length > 0 && line[length - 1] == '\n') {
       line[length - 1] = '\0';
     }
 
@@ -112,7 +112,7 @@ static ARRAY_T* read_scores_from_column
   fclose(score_file);
   int num_scores = i_score;
   if (verbosity >= NORMAL_VERBOSE) {
-    fprintf(stderr, "Read %d p-values from %s.\n", num_scores, 
+    fprintf(stderr, "Read %d values from %s.\n", num_scores,
 	    score_filename);
   }
 
@@ -133,7 +133,7 @@ static ARRAY_T* read_scores_from_column
  *
  * This is step (b) in Algorithm 3.1 of Storey (2002).
  *
- * The estimated pi_zero is 
+ * The estimated pi_zero is
  *
  *          W(\lambda)
  *        ---------------
@@ -158,7 +158,7 @@ static double estimate_pi_zero_fixed_lambda
   int num_greater = num_pvalues - start;
 
   // Compute pi_zero.
-  double pi_zero = (double)num_greater / 
+  double pi_zero = (double)num_greater /
     ((1.0 - lambda) * (double)num_pvalues);
 
   // If no p-values exceeded the given lambda, we have no estimate for pi_zero.
@@ -183,7 +183,7 @@ static double estimate_pi_zero_fixed_lambda
  * regarding this estimation procedure.
  *
  * "The paper shows that the estimate of pi_0 will always be biased high.
- * 
+ *
  * "A careful reading also will convince one that E[pFDR] in the
  * algorithm is proportional to pi_0, for a fixed lambda.  (This
  * explains why your algorithm works only in terms of pi_0, whereas
@@ -200,17 +200,23 @@ static double estimate_pi_zero_fixed_lambda
  * in any depth."
  *****************************************************************************/
 #define MIN_SAMPLES 100            // Minimum number of p-values required.
+
 double estimate_pi_zero
-  (int      num_bootstraps,        // How many bootstraps to perform.
+  (char*    pi_zero_filename,      // Name of file to store intermediate values.
+   int      num_bootstraps,        // How many bootstraps to perform.
    int      num_bootstrap_samples, // Number of p-values in each bootstrap.
    int      num_lambda,            // Number of lambda values to consider.
    float    max_lambda,            // Maximum lambda value to consider.
    ARRAY_T* p_values)
 {
+  if (verbosity >= NORMAL_VERBOSE) {
+    fprintf(stderr, "  Estimating pi_0.\n");
+  }
+  
   int num_pvalues = get_array_length(p_values);
 
   // Sort the p-values in ascending order.
-  sort_array(FALSE, p_values);
+  sort_array(false, p_values);
 
   // Allocate various arrays.
   ARRAY_T* pi_zeroes = allocate_array(num_lambda);
@@ -219,7 +225,7 @@ double estimate_pi_zero
   // Step through the requested lambdas.
   int i_lambda;
   for (i_lambda = 0; i_lambda < num_lambda; i_lambda++) {
-    double lambda = ((double)(i_lambda + 1) / (double)num_lambda) * MAX_LAMBDA;
+    double lambda = ((double)(i_lambda + 1) / (double)num_lambda) * max_lambda;
 
     // Get a simple estimate of pi_zero with a fixed lambda.
     double pi_zero = estimate_pi_zero_fixed_lambda(lambda, p_values);
@@ -243,20 +249,20 @@ double estimate_pi_zero
     }
 
     // Create an array of bootstrapped p-values.
-    ARRAY_T* bootstrapped_pvalues = bootstrap_array(p_values, 
+    ARRAY_T* bootstrapped_pvalues = bootstrap_array(p_values,
 						    num_bootstrap_samples);
-    sort_array(FALSE, bootstrapped_pvalues);
+    sort_array(false, bootstrapped_pvalues);
 
     for (i_lambda = 0; i_lambda < num_lambda; i_lambda++) {
-      double lambda = ((double)(i_lambda + 1) 
-		       / (double)num_lambda) * MAX_LAMBDA;
+      double lambda = ((double)(i_lambda + 1)
+		       / (double)num_lambda) * max_lambda;
 
       // Get a simple estimate of pi_zero with a fixed lambda.
-      double pi_zero = estimate_pi_zero_fixed_lambda(lambda, 
+      double pi_zero = estimate_pi_zero_fixed_lambda(lambda,
 						     bootstrapped_pvalues);
 
       // Estimated mean-squared error.
-      // See Equation (28) in Storey (2002). 
+      // See Equation (28) in Storey (2002).
       double error = pi_zero - min_pi_zero;
       double squared_error = error * error;
       incr_array_item(i_lambda, squared_error, mean_squared_errors);
@@ -281,12 +287,31 @@ double estimate_pi_zero
 
   // Use the corresponding pi_zero.
   double pi_zero = get_array_item(best_lambda_index, pi_zeroes);
-  
+
   // Ensure the pi_zero estimate is in the right range.
   pi_zero = MAX(MIN(pi_zero, 1.0), 0.0);
 
   if (verbosity >= NORMAL_VERBOSE) {
-    fprintf(stderr, "Estimated pi_0=%g\n", pi_zero);
+    fprintf(stderr, "  Estimated pi_0=%g\n", pi_zero);
+  }
+
+  // If requested, store the estimated pi-zero as a function of lambda.
+  if (pi_zero_filename != NULL) {
+    FILE* pi_zero_file = NULL;
+    if (open_file(pi_zero_filename, "w", true, "pi-zero", "pi-zero",
+		&pi_zero_file) == 0) {
+      exit(1);
+    }
+    fprintf(pi_zero_file, "p-value threshold\tlocal pi-zero\tfinal pi-zero\n");
+    for (i_lambda = 0; i_lambda < num_lambda; i_lambda++) {
+      double lambda = ((double)(i_lambda + 1) 
+		       / (double)num_lambda) * max_lambda;
+      fprintf(pi_zero_file, "%g\t%g\t%g\n", 
+	      lambda,
+	      get_array_item(i_lambda, pi_zeroes),
+	      pi_zero);
+    }
+    fclose(pi_zero_file);
   }
 
   // Free local dynamic memory.
@@ -300,24 +325,26 @@ double estimate_pi_zero
  * be in sorted (ascending) order.  This function replaces them with
  * the corresponding q-values.
  *
- * The total_values parameter may be larger than the number of values
- * stored in the array of p-values, but then pi_zero cannot be estimated.
- *
- * N.B. The max_lambda parameter is currently ignored.  The code uses
- * the MAX_LAMBDA macro instead.  It turns out that the results change
- * significantly if this is used as a variable rather than a macro.
+ * In order to estimate pi0 we need to have the distribution of pvalues.
+ * We can get the distribution from the pvalues array only if it contains
+ * all the observed pvalues. If we don't have all the observed p-values
+ * we can use the uniformly sampled subset of pvalues stored in the reservoir 
+ * sampler. If neither the full set of pvalues, or uniformly sampled pvalues
+ * are avilable, then we can't estimate pi0, and simply use pi0 = 1.0.
  *************************************************************************/
-void compute_qvalues
-  (BOOLEAN_T compute_fdr,      // If true, only compute FDR, not q-value.
-   BOOLEAN_T use_pi_zero,      // Estimate pi_zero; else just use 1.0.
-   char*     pi_zero_filename, // Filename to store pi-zero estimate in.
-   int       num_bootstraps,   // How many bootstraps to perform.
-   int       num_bootstrap_samples, // Number of p-values in each bootstrap.
-   int       num_lambda,      // Number of lambda values to consider.
-   float     max_lambda,       // Maximum lambda value to consider.
-   int       total_values,     // Total number of values.
-   ARRAY_T*  pvalues)
-{
+void compute_qvalues(
+  bool compute_fdr,      // If true, only compute FDR, not q-value.
+  bool use_pi_zero,      // Estimate pi_zero; else just use 1.0.
+  char*     pi_zero_filename, // Filename to store pi-zero estimate in.
+  int       num_bootstraps,   // How many bootstraps to perform.
+  int       num_bootstrap_samples, // Number of p-values in each bootstrap.
+  int       num_lambda,       // Number of lambda values to consider.
+  float     max_lambda,       // Maximum lambda value to consider.
+  long      total_values,     // Total number of p-values observed.
+  ARRAY_T*  pvalues,         // retained pvalues
+  ARRAY_T*  sampled_pvalues  // uniformly sampled pvalues
+) {
+
   // Make sure we got at least one p-value.
   int num_values = get_array_length(pvalues);
   if (num_values == 0) {
@@ -333,52 +360,93 @@ void compute_qvalues
   }
 
   // Verify that the p-values are in sorted order.
-  ATYPE prev_pvalue = get_array_item(0, pvalues);
-  for (i_pvalue = 1; i_pvalue < num_values; i_pvalue++) {
-    ATYPE this_pvalue = get_array_item(i_pvalue, pvalues);
-    myassert(1, this_pvalue >= prev_pvalue, "Unsorted p-values (%g < %g)\n",
-	     this_pvalue, prev_pvalue);
-    prev_pvalue = this_pvalue;
-  }
+  myassert(1, is_sorted(true, pvalues), "The p-values are not sorted.");
 
-  // Estimate pi_zero, if requested.
   double pi_zero = 1.0;
+  
+  // Estimate pi_zero, if requested.
   if (use_pi_zero) {
 
-    if (total_values != num_values) {
-      fprintf(stderr, "Warning: Cannot estimate pi_zero without the complete range of p-values.\n");
-      fprintf(stderr, "         Using pi_zero = 1.0.\n");
-
-    } else if (total_values < MIN_SAMPLES) {
-      fprintf(stderr, "Warning: Cannot estimate pi_zero accurately from fewer than %d p-values.\n", MIN_SAMPLES);
-      fprintf(stderr, "         Using pi_zero = 1.0.\n");
+    bool have_all_pvalues = (total_values == (long) num_values);
 
+    if (have_all_pvalues) {
+      if (num_values > (long) MIN_SAMPLES) {
+        // Estimate pi0 from complete set of pvalues
+        if (verbosity >= NORMAL_VERBOSE) {
+          fprintf(
+            stderr, 
+            "  Estimating pi_0 from all %d observed p-values.\n",
+            num_values
+          );
+        }
+        pi_zero = estimate_pi_zero(
+           pi_zero_filename,
+           num_bootstraps,
+           num_bootstrap_samples,
+           num_lambda,
+           max_lambda,
+           pvalues
+        );
+      } else {
+        if (verbosity >= NORMAL_VERBOSE) {
+          fprintf(
+            stderr, 
+            "Warning: Cannot estimate pi_0 accurately from fewer than %d p-values."
+            "         total p-values = %d. Using pi_zero = 1.0.\n",
+            MIN_SAMPLES,
+            num_values
+          );
+        }
+      }
+    } else if (sampled_pvalues != NULL) {
+      int num_samples = get_array_length(sampled_pvalues);
+      if (num_samples > MIN_SAMPLES) {
+        // Estimate pi0 from uniformly sampled pvalues
+        if (verbosity >= NORMAL_VERBOSE) {
+          fprintf(
+            stderr,
+            "  Estimating pi_0 from a uniformly sampled set "
+            "of %d p-values.\n",
+            num_samples
+          );
+        }
+        pi_zero = estimate_pi_zero(
+           pi_zero_filename,
+           num_bootstraps,
+           num_bootstrap_samples,
+           num_lambda,
+           max_lambda,
+           sampled_pvalues
+        );
+      } 
+      else {
+        fprintf(
+          stderr, 
+          "Warning: Cannot estimate pi_0 accurately from fewer "
+          "than %d sampled p-values.\n"
+          "         only %d p-values were sampled. Using pi_zero = 1.0.\n",
+          MIN_SAMPLES,
+          num_samples
+        );
+      }
     } else {
-      pi_zero = estimate_pi_zero(num_bootstraps,
-				 num_bootstrap_samples,
-				 num_lambda,
-				 max_lambda,
-				 pvalues);
+      fprintf(
+        stderr, 
+        "Warning: Cannot estimate pi_zero without the complete range of p-values,\n"
+        "         or a uniformly sampled subset of pvalues. Using pi_zero = 1.0.\n"
+      );
     }
   }
 
-  // If requested, store the estimated pi-zero.
-  if (pi_zero_filename != NULL) {
-    FILE* pi_zero_file = NULL;
-    if (open_file(pi_zero_filename, "w", TRUE, "pi-zero", "pi-zero",
-		&pi_zero_file) == 0) {
-      exit(1);
-    }
-    fprintf(pi_zero_file, "%g\n", pi_zero);
-    fclose(pi_zero_file);
-  }
-
-  // Compute FDRs.
+  // Traverse p-values from lowest to highest.
   for (i_pvalue = 0; i_pvalue < num_values; i_pvalue++) {
     ATYPE this_pvalue = get_array_item(i_pvalue, pvalues);
-    ATYPE this_fdr = (pi_zero * this_pvalue * (float)total_values) 
+
+    ATYPE this_fdr = (pi_zero * this_pvalue * (float)total_values)
       / (float)(i_pvalue + 1);
-    set_array_item(i_pvalue, MIN(1.0,this_fdr), pvalues);
+
+    // Can't have an FDR > 1.0.
+    set_array_item(i_pvalue, MIN(1.0, this_fdr), pvalues);
   }
 
   // No conversion if we only got one value or if only computing FDR.
@@ -387,7 +455,7 @@ void compute_qvalues
   }
 
   // Convert to q-values.
-  prev_pvalue = get_array_item(num_values - 1, pvalues);
+  ATYPE prev_pvalue = get_array_item(num_values - 1, pvalues);
   for (i_pvalue = num_values - 2; i_pvalue >= 0; i_pvalue--) {
     ATYPE this_pvalue = get_array_item(i_pvalue, pvalues);
     if (prev_pvalue < this_pvalue) {
@@ -400,41 +468,50 @@ void compute_qvalues
 /*****************************************************************************
  * Convert scores to p-values using an empirical null.
  *
- * The p-value associated with an observed score x is simply (a+1)/(b+1),
- * where a is the number of null scores > x, and b is the total number
- * of null scores.
+ * The p-value associated with an observed score x is simply
+ * (a+1)/(b+1), where a is the number of null scores better than x,
+ * and b is the total number of null scores.
  *
  * The +1 correction is recommended in the article, "A Note on the
  * Calculation of Empirical P Values from Monte Carlo Procedures."
  * http://www.pubmedcentral.nih.gov/articlerender.fcgi?artid=379178
  *****************************************************************************/
 void convert_scores_to_pvalues
-(ARRAY_T* observed_scores, 
- ARRAY_T* null_scores)
+  (bool good_score_is_low,
+   ARRAY_T*  observed_scores,
+   ARRAY_T*  null_scores)
 {
-  // Sort both arrays in ascending order.
-  sort_array(FALSE, observed_scores);
-  sort_array(FALSE, null_scores);
+  if (verbosity >= NORMAL_VERBOSE) {
+    fprintf(stderr, "Converting scores to p-values.\n");
+  }
+
+  // Sort both arrays in order, best to worst.
+  sort_array(good_score_is_low != true, observed_scores);
+  sort_array(good_score_is_low != true, null_scores);
 
   int num_observed = get_array_length(observed_scores);
   int num_null = get_array_length(null_scores);
 
-  // Get the first null score.
+  // Get the best null score.
   int i_null = 0;
   ATYPE null_score = get_array_item(i_null, null_scores);
 
-  // Traverse the observed scores, computing a p-value for each.
+  // Traverse the observed scores, best to worst.
   int i_observed;
   for (i_observed = 0; i_observed < num_observed; i_observed++) {
     ATYPE observed_score = get_array_item(i_observed, observed_scores);
 
-    while ((observed_score > null_score) && (i_null < num_null)) {
+    // Find the first null score that is worse than the observed score.
+    while ((((good_score_is_low == true) && (observed_score > null_score)) ||
+	    ((good_score_is_low == false) && (observed_score < null_score))) &&
+	   (i_null < num_null)) {
       i_null++;
       if (i_null < num_null) {
 	null_score = get_array_item(i_null, null_scores);
       }
     }
-    ATYPE pvalue = (ATYPE)((num_null - i_null) + 1) / (ATYPE)(num_null + 1);
+
+    ATYPE pvalue = (ATYPE)(i_null + 1) / (ATYPE)(num_null + 1);
     set_array_item(i_observed, pvalue, observed_scores);
   }
 }
@@ -454,19 +531,21 @@ int main
 
   // Default parameter settings.
   char*     null_filename = NULL; // By default, assume p-value input.
-  BOOLEAN_T compute_fdr = FALSE;  // By default, compute p-value.
-  BOOLEAN_T use_pi_zero = FALSE; // By default, compute q-value, not just FDR.
+  bool good_score_is_low = true; // By default, good score is low.
+  bool compute_fdr = false;  // By default, compute p-value.
+  bool use_pi_zero = false; // By default, compute q-value, not just FDR.
   char*     pi_zero_filename = NULL; //  Store pi-zero in the given file.
   int       num_bootstraps = NUM_BOOTSTRAPS;
   int       num_header_lines = 0;
   int       pvalue_column = 0; // Indexed from 0.
-  BOOLEAN_T append_output = FALSE; // Append q-values to end of input line?
+  bool append_output = false; // Append q-values to end of input line?
   long      seed = time(0);
   verbosity = NORMAL_VERBOSE;
 
-  const int num_options = 9;
+  const int num_options = 11;
   cmdoption const options[] = {
     { "null", REQUIRED_VALUE },
+    { "good-score", REQUIRED_VALUE },
     { "pi-zero", NO_VALUE },
     { "pi-zero-file", REQUIRED_VALUE },
     { "fdr", NO_VALUE},
@@ -480,10 +559,11 @@ int main
 
   // Define the usage message.
   char      usage[400] = "";
-  strcat(usage, "USAGE: qvalue [options] <pvalues>\n");
+  strcat(usage, "Usage: qvalue [options] <pvalues>\n");
   strcat(usage, "\n");
   strcat(usage, "   Options:\n");
   strcat(usage, "     --null <file>\n");
+  strcat(usage, "     --good-score high|low\n");
   strcat(usage, "     --pi-zero\n");
   strcat(usage, "     --pi-zero-file <file>\n");
   strcat(usage, "     --fdr\n");
@@ -513,13 +593,23 @@ int main
 
     if (strcmp(option_name, "null") == 0) {
       null_filename = option_value;
+    } else if (strcmp(option_name, "good-score") == 0) {
+      if (strcmp(option_value, "high") == 0) {
+	good_score_is_low = false;
+      } else if (strcmp(option_value, "low") == 0) {
+	good_score_is_low = true;
+      } else {
+	fprintf(stderr, "Invalid option (--good-score %s)\n",
+		option_value);
+	exit(1);
+      }
     } else if (strcmp(option_name, "pi-zero") == 0) {
-      use_pi_zero = TRUE;
+      use_pi_zero = true;
     } else if (strcmp(option_name, "pi-zero-file") == 0) {
-      use_pi_zero = TRUE;
+      use_pi_zero = true;
       pi_zero_filename = option_value;
     } else if (strcmp(option_name, "fdr") == 0) {
-      compute_fdr = TRUE;
+      compute_fdr = true;
     } else if (strcmp(option_name, "bootstraps") == 0) {
       num_bootstraps = atoi(option_value);
     } else if (strcmp(option_name, "header") == 0) {
@@ -527,7 +617,7 @@ int main
     } else if (strcmp(option_name, "column") == 0) {
       pvalue_column = atoi(option_value) - 1; // User inputs indexed from 1.
     } else if (strcmp(option_name, "append") == 0) {
-      append_output = TRUE;
+      append_output = true;
     } else if (strcmp(option_name, "seed") == 0) {
       seed = atoi(option_value);
     } else if (strcmp(option_name, "verbosity") == 0) {
@@ -537,25 +627,29 @@ int main
 
   // Read the single required argument.
   if (option_index + 1 != argc) {
-    fprintf(stderr, usage);
+    fprintf(stderr, "%s", usage);
     exit(1);
   }
   char* pvalue_filename = argv[option_index];
 
   // Initialize the random number generator.
-  my_srand(seed);
+  srand_mt(seed);
 
   // Make a string list to store the file contents.
   STRING_LIST_T* input_lines = NULL;
+  STRING_LIST_T* header_lines = NULL;
   if (append_output) {
     input_lines = new_string_list();
+    if (num_header_lines != 0) {
+      header_lines = new_string_list();
+    }
   }
 
   // Read the p-values from the specified column.
   ARRAY_T* pvalues = read_scores_from_column(pvalue_filename,
 					     num_header_lines,
-					     TRUE, // Print header
 					     pvalue_column,
+					     header_lines,
 					     input_lines);
   int num_pvalues = get_array_length(pvalues);
 
@@ -563,26 +657,28 @@ int main
   ARRAY_T* raw_scores = NULL;
   if (null_filename != NULL) {
 
-    // Store a copy of the raw scores for later reference.
+    // Store a copy of the raw scores for later reference, best score first.
     raw_scores = allocate_array(num_pvalues);
     copy_array(pvalues, raw_scores);
-    sort_array(TRUE, raw_scores);
+    sort_array(!good_score_is_low, raw_scores);
 
     // Read the null scores.
     ARRAY_T* null_scores = read_scores_from_column(null_filename,
 						   num_header_lines,
-						   FALSE, // Do not print header
 						   pvalue_column,
+						   NULL, // Do not store
 						   NULL); // Do not store
 
     // Compute p-values.
-    convert_scores_to_pvalues(pvalues, null_scores);
+    convert_scores_to_pvalues(good_score_is_low, pvalues, null_scores);
   }
 
-  // Sort the p-values.
-  sort_array(FALSE, pvalues);
-  if (append_output == TRUE) {
-    sort_string_list_by_score(input_lines, FALSE);
+  // Sort the p-values, if necessary.
+  if (!is_sorted(true, pvalues)) {
+    sort_array(false, pvalues);
+    if (append_output == true) {
+      sort_string_list_by_score(input_lines, good_score_is_low != true);
+    }
   }
 
   // Make a copy of the p-values.
@@ -590,7 +686,8 @@ int main
   copy_array(pvalues, qvalues);
 
   // Compute q-values.
-  compute_qvalues(compute_fdr, 
+  compute_qvalues(
+      compute_fdr,
 		  use_pi_zero,
 		  pi_zero_filename,
 		  num_bootstraps,
@@ -598,22 +695,45 @@ int main
 		  NUM_LAMBDA,
 		  MAX_LAMBDA,
 		  get_array_length(qvalues),
-		  qvalues);
+		  qvalues,
+      NULL
+    );
+
+  // Print header.
+  if (append_output) {
+    int i_header = 0;
+    for (i_header = 0; i_header < num_header_lines; i_header++) {
+      if (raw_scores != NULL) {
+	printf("%s\tp-value\tq-value\n",
+	       get_nth_string(i_header, header_lines));
+      } else {
+	printf("%s\tq-value\n",
+	       get_nth_string(i_header, header_lines));
+      }
+    }
+  }
 
   // Print to stdout.
   int i_pvalue;
   for (i_pvalue = 0; i_pvalue < num_pvalues; i_pvalue++) {
     if (append_output) {
-      printf("%s\t%g\n", 
-	     get_nth_string(i_pvalue, input_lines),
-	     get_array_item(i_pvalue, qvalues));
+      if (raw_scores != NULL) {
+	printf("%s\t%g\t%g\n",
+	       get_nth_string(i_pvalue, input_lines),
+	       get_array_item(i_pvalue, pvalues),
+	       get_array_item(i_pvalue, qvalues));
+      } else {
+	printf("%s\t%g\n",
+	       get_nth_string(i_pvalue, input_lines),
+	       get_array_item(i_pvalue, qvalues));
+      }
     } else if (raw_scores != NULL) {
-      printf("%g\t%g\t%g\n", 
+      printf("%g\t%g\t%g\n",
 	     get_array_item(i_pvalue, raw_scores),
 	     get_array_item(i_pvalue, pvalues),
 	     get_array_item(i_pvalue, qvalues));
     } else {
-      printf("%g\t%g\n", 
+      printf("%g\t%g\n",
 	     get_array_item(i_pvalue, pvalues),
 	     get_array_item(i_pvalue, qvalues));
     }
