@@ -8,15 +8,14 @@
  ********************************************************************/
 #ifndef UTILS_H
 #define UTILS_H
+#include <stdbool.h>
 #include <stdio.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <math.h>
+#include <float.h>
 #include "macros.h"
 
-#define FALSE 0
-#define TRUE 1
-typedef short BOOLEAN_T;
-
 typedef int VERBOSE_T;
 #define INVALID_VERBOSE 0
 #define QUIET_VERBOSE 1
@@ -27,25 +26,47 @@ typedef int VERBOSE_T;
 
 #define BIG HUGE_VAL
 #define LITTLE -BIG
+#define SMALL_POS 1e-300
+
+#define SHUFFLE(base, n) shuffle(base, n, sizeof *(base))
 
 extern VERBOSE_T verbosity;
 
-/***********************************************************************
- * Return a not-a-number.
- ***********************************************************************/
-double NaN
-  (void);
+/*
+ * Support LLVM as a compiler.
+ */
+#ifdef __llvm__
+#define inline
+#endif
 
 /********************************************************************
  * double myclock
  *
- * Return number of CPU microseconds since first call to myclock().
+ * Return number of user microseconds since first call to myclock().
  * This corrects the bug in the system version of clock that causes it
  * to loop after about 36 minutes.
  *
  * (Taken from Tim Bailey's MEME package.)
  ********************************************************************/
 double myclock(void);
+/********************************************************************
+ * double mysysclock
+ *
+ * Return number of system microseconds since first call to my_sys_clock().
+ * This corrects the bug in the system version of clock that causes it
+ * to loop after about 36 minutes.
+ *
+ * (Taken from Tim Bailey's MEME package.)
+ ********************************************************************/
+double mysysclock(void);
+
+//
+// Return the elapsed Wall Time in microseconds since
+// the first call to mytime() (or the previous call).
+//
+double mytime(
+  bool previous		// if True, time since previous call, not first call
+);
 
 /************************************************************************
  * char* make_path_to_file
@@ -75,12 +96,12 @@ char* concat_string( const char *string1, const char* string2);
  *
  * RETURN: Was the open successful?
  ************************************************************************/
-BOOLEAN_T open_file
-  (char*     filename,            /* Name of the file to be opened. */
-   char*     file_mode,           /* Mode to be passed to fopen. */
-   BOOLEAN_T allow_stdin,         /* If true, filename "-" is stdin. */
-   char*     file_description,
-   char*     content_description,
+bool open_file
+  (const char*     filename,            /* Name of the file to be opened. */
+   const char*     file_mode,           /* Mode to be passed to fopen. */
+   bool allow_stdin,         /* If true, filename "-" is stdin. */
+   const char*     file_description,
+   const char*     content_description,
    FILE **         afile);              /* Pointer to the open file. */
 
 /*************************************************************************
@@ -107,7 +128,7 @@ FILE* open_command_pipe
    char*     test_arguments,   // Arguments used when searching for program.
    char*     expected_reply,   // Expected reply from search.
    char*     real_arguments,   // Arguments used when running the program.
-   BOOLEAN_T stdout_on_error,  // If command fails, return STDOUT?
+   bool stdout_on_error,  // If command fails, return STDOUT?
    char*     error_message);   // Error or warning if command fails.
 
 
@@ -139,14 +160,14 @@ void die
 /**************************************************************************
  * Make an assertion, and print the given message if the assertion fails.
  *
- * If the first parameter is set to TRUE, then die if the assertion
+ * If the first parameter is set to true, then die if the assertion
  * doesn't go through.  Otherwise, just issue the warning.
  *
  * On exit, dump core if DEBUG is defined.
  **************************************************************************/
 void myassert
-  (BOOLEAN_T die_on_error,
-   BOOLEAN_T test,
+  (bool die_on_error,
+   bool test,
    char*  const    format,
    ...);
 
@@ -176,18 +197,6 @@ void * mm_realloc
   (P) = (T *) new_P; \
 }
 
-/********************************************************************
- * Set the seed for the random number generator.
- ********************************************************************/
-void my_srand
-  (long seed);
-
-/********************************************************************
- * Get a random number X such that 0 <= X < 1.
- ********************************************************************/
-double my_drand
-  (void);
-
 /********************************************************************
  * Math macros.
  ********************************************************************/
@@ -223,13 +232,11 @@ PROB_T log_prob
 
 #define log_prob2(x)   (log_prob(x) * 1.44269504)
 
-/**************************************************************************
- * Compute the logarithm of x.  Returns LOG_ZERO if x==0.
- **************************************************************************/
-PROB_T my_log
-  (PROB_T x);
-
-#define my_log2(x)   (my_log(x) * 1.44269504)
+#define LOG_VALUE(logx) \
+( ( (logx) < LOG_SMALL ) ? \
+    LOG_ZERO : \
+    (logx) \
+)
 
 #define EXP2(x)                          \
 ( ( (x) < LOG_SMALL) ?                   \
@@ -237,6 +244,28 @@ PROB_T my_log
   (exp((x) * 0.69314718 ))               \
 )
 
+/**************************************************************************
+ * Compute the logarithm of x.  Returns SMALL_POS if x==0.
+ **************************************************************************/
+static inline PROB_T my_log_helper(PROB_T x, bool base2) {
+  double lx;
+  if (x > 0.0) {
+    lx = log(x);
+    if (base2) {
+      return(LOG_VALUE(lx) * 1.44269504);
+    } else {
+      return(LOG_VALUE(lx));
+    }
+  } else if (x < 0.0) {
+    die("Tried to take the log of a negative value (%g).", x);
+  } // else if (x == 0.0)
+  return(SMALL_POS);
+}
+
+#define my_log(x)   my_log_helper(x, false)
+#define my_log2(x)  my_log_helper(x, true)
+
+
 /**************************************************************************
  * Given the logs (in base 2) of two numbers, return the log of their
  * sum.
@@ -246,11 +275,6 @@ PROB_T my_log
  *      log(x+y) = log(x) + log(1 + exp(log(y) - log(x)))
  *
  **************************************************************************/
-#define LOG_VALUE(logx) \
-( ( (logx) < LOG_SMALL ) ? \
-    LOG_ZERO : \
-    (logx) \
-)
 
 #define LOG_SUM1(logx, logy) \
 ( \
@@ -266,17 +290,27 @@ PROB_T my_log
     LOG_SUM1( (logy), (logx) ) \
 )
 
+/**************************************************************************
+ * Return the nearest double smaller than the given double
+ **************************************************************************/
+double get_next_smaller_double(double x);
+
+/**************************************************************************
+ * Return the nearest double larger than the given double
+ **************************************************************************/
+double get_next_larger_double(double x);
+
 /**************************************************************************
  * Test for zero on a value that may be either a log or a raw float.
  **************************************************************************/
-BOOLEAN_T is_zero
+bool is_zero
   (double    value,
-   BOOLEAN_T log_form);
+   bool log_form);
 
 /**************************************************************************
  * Test to see if two values are approximately equal.
  **************************************************************************/
-BOOLEAN_T almost_equal
+bool almost_equal
   (double value1,
    double value2,
    double slop);
@@ -284,16 +318,73 @@ BOOLEAN_T almost_equal
 /*************************************************************************
  * Convert a boolean to and from a "true" or "false" string.
  *************************************************************************/
-char*  boolean_to_string
- (BOOLEAN_T the_boolean);
+const char*  boolean_to_string(bool the_boolean);
 
-BOOLEAN_T boolean_from_string
-  (char* true_or_false);
+bool boolean_from_string(char* true_or_false);
+
+/*************************************************************************
+ * Writes a unicode codepoint in UTF-8 encoding to the start of the buffer
+ * and return it, optionally record the length of the written code unit.
+ * The buffer must be at least 6 bytes long to hold any valid codepoint
+ * though it will use the minimum possible. The output is NOT null terminated.
+ *************************************************************************/
+char* unicode_to_string(uint32_t code, char *buffer, int *code_unit_length);
+
+/*************************************************************************
+ * Returns the Unicode codepoint at the start of the string assuming UTF-8.
+ * This function handles NUL bytes!
+ *
+ * If something is wrong it will return one of these error codes:
+ * -1   If the string begins with a byte 10xxxxxx indicating a middle byte,
+ * -2   If the codepoint would go past the end of the string,
+ * -3   If the start byte is the illegal value 0xFE, or 0xFF
+ * -4   If there are too few middle bytes following the start byte,
+ * -5   If the codepoint uses more bytes than needed.
+ *************************************************************************/
+int32_t unicode_from_string(const char *str, size_t len, int *code_unit_length);
+
+/*************************************************************************
+ * MEME and DREME motifs can have very small E-values which are impossible
+ * to represent using a double. By converting to log values we lose 
+ * precision but the range possible becomes much greater.
+ *************************************************************************/
+double log10_evalue_from_string(const char *str);
+
+/*************************************************************************
+ * MEME and DREME motifs can have very small E-values which are impossible
+ * to represent using a double. Internally we represent such small evalues
+ * as log evalues. When outputting we have to convert those log values
+ * back into a string representation taking into account the possiblity
+ * that the user specified an e-value of zero (which we actually recommended
+ * in documentation for quite a while).
+ *
+ * This function writes the E-value to a string buffer in scientific
+ * notation.
+ *
+ * This returns the number of characters printed (excluding the nul byte).
+ * If the returned value is larger than or equal to size then it has
+ * been truncated and not nul terminated.
+ *************************************************************************/
+int log10_evalue_to_string(double log10_ev, int prec, char *buffer, int size);
+
+/*************************************************************************
+ * MEME and DREME motifs can have very small E-values which are impossible
+ * to represent using a double. Internally we represent such small evalues
+ * as log evalues. When outputting we have to convert those log values
+ * back into a string representation taking into account the possiblity
+ * that the user specified an e-value of zero (which we actually recommended
+ * in documentation for quite a while).
+ *
+ * This function writes the E-value to an expandable string buffer in scientific
+ * notation. The buffer, expanded to fit the number if required, is returned.
+ *************************************************************************/
+char* log10_evalue_to_string2(double log10_ev, int prec,
+    char **expandable_buffer, int *buffer_size, int offset);
 
 /**************************************************************************
  * Does a given character appear in a given string?
  **************************************************************************/
-BOOLEAN_T char_in_string
+bool char_in_string
   (const char* a_string,
    char        a_char);
 
@@ -340,13 +431,13 @@ char *get_last_modified_time(char *filename, char *date_buffer);
  **************************************************************************/
 void copy_string
  (char**  target,
-  char*   source);
+  const char*   source);
 
 /************************************************************************
  * Test whether a string is empty or contains only white space
  * Assumes a null terminated string.
  ************************************************************************/
-BOOLEAN_T is_empty_string
+bool is_empty_string
   (char *s);
 
 /************************************************************************
@@ -368,28 +459,84 @@ void get_non_blank
 /************************************************************************
  *  Is the next character in the file the eoln character
  ************************************************************************/
-BOOLEAN_T is_eoln
+bool is_eoln
   (FILE * infile);
 
 /************************************************************************
  *  Does a file exist?
  ************************************************************************/
-BOOLEAN_T file_exists(char* file_path);
+bool file_exists(char* file_path);
+
+/************************************************************************
+ *  Is a file executable?
+ ************************************************************************/
+bool file_executable(char* file_path);
+
+/************************************************************************
+ *  Is a file a directory?
+ ************************************************************************/
+bool file_directory(char* file_path);
 
 /******************************************************************
  * This function copies the contents of the file named by the first
  * argument into the file named by the second argument.
  *
- * Returns TRUE if successful, FALSE otherwise.
+ * Returns true if successful, false otherwise.
 ******************************************************************/
-BOOLEAN_T copy_file(char *from_filename, char *to_filename);
+bool copy_file(char *from_filename, char *to_filename);
 
 /******************************************************************
  * This function checks the environment for a variable pointing
- * to where the MEME files are installed.
- *
+ * to where the MEME data files are installed. If the environment
+ * variable is not found, it returns the definitions from the configuration.
 ******************************************************************/
-const char* get_meme_etc_dir();
+const char* get_meme_data_dir();
+
+/******************************************************************
+ * This function checks the environment for a variable pointing
+ * to where the MEME executable files are installed. If the environment
+ * variable is not found it returns the definition from the configuration.
+******************************************************************/
+const char* get_meme_bin_dir();
+
+/******************************************************************
+ * This function checks the environment for a variable pointing
+ * to where the MEME temporary files should be placed. If the environment
+ * variable is not found it returns the definition from the configuration
+ * and if that is not given then it searches the environment variables
+ * TMPDIR, TMP, TEMP and TEMPDIR. If none of those exist it defaults
+ * to "/tmp"
+******************************************************************/
+const char* get_meme_temp_dir();
+
+/******************************************************************
+ * This function checks the colon separated dirs for a file with
+ * the given name.
+ * If the file is found then an allocated path is returned to it,
+ * alternatively NULL is returned if no such file exists.
+ * The caller is responsible for freeing memory.
+******************************************************************/
+char* get_meme_dirs_file(const char* dirs, const char* file_name);
+
+/******************************************************************
+ * This function checks the environment variables MEME_BIN_DIRS and
+ * MEME_BIN_DIR for a file with the given name. If neither of those
+ * is set then it will look for the file in the compiled BIN_DIR.
+ * If the file is found then an allocated path is returned to it,
+ * alternatively NULL is returned if no such file exists.
+ * The caller is responsible for freeing memory.
+******************************************************************/
+char* get_meme_bin_file(const char* file_name);
+
+/******************************************************************
+ * This function checks the environment variables MEME_BIN_DIRS and
+ * MEME_LIBEXEC_DIR for a file with the given name. If neither of those
+ * is set then it will look for the file in the compiled LIBEXEC_DIR.
+ * If the file is found then an allocated path is returned to it,
+ * alternatively NULL is returned if no such file exists.
+ * The caller is responsible for freeing memory.
+******************************************************************/
+char* get_meme_libexec_file();
 
 /******************************************************************
  *
@@ -399,5 +546,60 @@ const char* get_meme_etc_dir();
 ******************************************************************/
 char *get_command_line(int argc, char* argv[]);
 
-#endif
+/******************************************************************
+ * Get a description from a file or a string.
+ *
+ * If there is a file containing the description then read up to
+ * 500 characters from the file.
+ * If there is a string, then copy it.
+ * If neither a file or string is avaliable then return null.
+ * Convert windows new lines (CRLF) into into line feed. Convert old mac
+ * new lines (CR) into line feed, note that LFCR is not supported. 
+ * Merge 3 or more contiguous line feeds into two line feeds.
+ * Remove leading and trailing space.
+ *
+ * The caller is responsible for freeing the returned string.
+******************************************************************/
+char *get_job_description(const char* desc_file, const char* desc_message);
 
+/******************************************************************
+ * Randomly shuffle the elements of an array.
+******************************************************************/
+void shuffle(void *base, size_t n, size_t size);
+
+/* Initialize Mersenne Twist random number generator. */
+void srand_mt(
+  uint32_t seed
+);
+
+/* Random double in [0.0, 1.0); Mersenne Twist */
+/* Uses program-wide state. */
+double drand_mt();
+
+/* Random integer between 0 (inclusive) and 2^31-1; Mersenne Twist */
+long random_mt();
+
+/* Random integer between 0 (inclusive) and n (exclusive) */
+/* n must be > 0 and <= RAND_MAX+1; Mersenne Twist */
+int rand_int(const unsigned n);
+
+/* Random double between 0 (inclusive) and n (exclusive) */
+/* n must be > 0; Mersenne Twist */
+double rand_dbl(const double n);
+
+/*****************************************************************************
+ * Outputs the file name when given a path to that file.
+ *
+ * Searches the path string for '/' chars.  Removes the extension
+ * if requested.  Replaces underscores with spaces if requested.
+ *
+ * This is not intended to be perfect as I'm sure there's some valid way to have
+ * a '/' in a file name but I expect it will work most of the time.
+ *****************************************************************************/
+char *file_name_from_path(
+  char *path, 
+  bool remove_ext, 
+  bool remove_meme_ext,
+  bool replace_underscores
+);
+#endif
